@@ -84,25 +84,38 @@ typedef int* pint;
        happens next is specified in the wget documentation (I guess).
 */
 
-Downloader::Downloader(QString directory)
+
+// only works if loaded in setup plugin
+Downloader::Downloader(QString directory, StreamStorage *storage)
 {
    this->directory = directory;
+   this->home = getenv("HOME");
+   
    downloads.clear();
    umask(2);  // read/write rights for user and group
+   streamStorage = storage;
+       connect(
+             streamStorage,
+             SIGNAL(recordRemoved(ChangedRecord*)),
+             this,
+             SLOT(slotRecordRemoved(ChangedRecord*)) );
+
 }
 
-QString Downloader::download(QString url, bool& canplaynow, bool& error, QString& errorMessage)
+QString Downloader::download(QString url, QString sName, bool& canplaynow, bool& error,
+                             bool& itemcreated, QString& errorMessage, bool checkName)
 // returns absolute path of downloading file
 {
    QString path;
    Availability availability;
-   bool pathFailed;
+   bool pathFailed = false;
 
    error        = false;
    errorMessage = "";
    canplaynow   = false;
+   itemcreated  = false;
    
-   path = checkAvailable(url, availability, pathFailed, true);
+   path = checkAvailable(url, availability, pathFailed, true, sName, checkName);
 
    // todo: when [filename]_temp file exists and no download on that file
    // is active, it should resume (wget -c). But Wget refuses to download if
@@ -114,8 +127,10 @@ QString Downloader::download(QString url, bool& canplaynow, bool& error, QString
    
    canplaynow = ( availability != notavailable );
    if (pathFailed)
-     errorMessage = TARGET" error: could not create file " + path;
+     errorMessage = " error: could not create file " + path;
 
+   error = pathFailed;
+   
    if (availability == notavailable && !pathFailed)
    {
         proc = new QProcess(this);
@@ -140,7 +155,10 @@ QString Downloader::download(QString url, bool& canplaynow, bool& error, QString
           delete proc;
         }
           else
+        {
             downloads.append(url);
+            itemcreated = createStreamItem(sName, path, url, "");  // filename passed as url
+        }
 
    }
 
@@ -173,8 +191,9 @@ void Downloader::slotDownloadStopped()
          if ( path.right(5) == "_temp" )
          {
             QString newPath = path.left( path.length() - 5 );
-            QDir dir(directory);
+            QDir dir(home);
             dir.rename(path, newPath, true);
+            updateStreamItemFilename(path, newPath);
          }
       }  // else leave file for download resume/overwrite on next select
         else
@@ -191,7 +210,8 @@ void Downloader::slotDownloadStopped()
 }
 
 QString Downloader::checkAvailable( QString url, Availability& availability,
-                                    bool& errorWhileCreating, bool createPathIfNotExists )
+                                    bool& errorWhileCreating, bool createPathIfNotExists,
+                                    QString name, bool checkName)
 {
    QString path;
    errorWhileCreating = false;
@@ -200,8 +220,12 @@ QString Downloader::checkAvailable( QString url, Availability& availability,
    errorWhileCreating = false;
           
    QString urlPath = url.section( "://", 1, 1);  // strip protocol
+ 
+   // change "../" to "xx/".  This prevents any unintended directory traversal.
+   urlPath.replace("../", "xx/");
+   
    path = directory + "/" + urlPath;
-   QFile file(path);
+   QFile file(home + "/" + path);
 
    if ( file.exists() )
    {
@@ -211,7 +235,7 @@ QString Downloader::checkAvailable( QString url, Availability& availability,
 
    path    += "_temp";
    urlPath += "_temp";
-   file.setName(path);
+   file.setName(home + "/" + path);
 
    if ( file.exists() )
    {
@@ -219,6 +243,27 @@ QString Downloader::checkAvailable( QString url, Availability& availability,
      return path;
    }
 
+ // should check host part of url (that should be, but is not?) in descr...
+   if (checkName)
+   {
+		bool found = false;
+		
+		streamStorage->resetRecordList();
+
+		ValueList values(5);
+		while ( !found && streamStorage->getNextRecord(values) )
+		{
+			if ( values.count() == 5 && values[s_folder] == "downloads" && values[s_name] == name )
+			found = true;
+		}
+
+		if (found)
+		{
+         availability = available;
+         return values[s_url];
+		}
+   }
+
    if ( !createPathIfNotExists )
      return "";
 
@@ -232,17 +277,108 @@ QString Downloader::checkAvailable( QString url, Availability& availability,
    {
       currentDir += "/" + *pathElements.at(i);
       //cout << "create: " << currentDir << endl;
-      dir = QDir( currentDir );
-
+      dir = QDir( home + "/" + currentDir );
+      
       if ( !dir.exists() )
-        pathCreated = dir.mkdir(currentDir);
+      {
+        pathCreated = dir.mkdir(home + "/" + currentDir);
+        if (!pathCreated) cout << TARGET" error: cannot create " << currentDir << endl;
+      }
    }
 
    errorWhileCreating = !pathCreated;
    
+   if (errorWhileCreating)
+     cout << TARGET" error: path create error for " << urlPath << endl;
+   
    return path;
 }
 
+
+bool Downloader::createStreamItem(QString name, QString url, QString descr, QString handler)
+{
+    QString errorMsg;
+    ValueList values(5);
+    values[s_folder] = "downloads";
+    values[s_name]   = name;
+    values[s_url]    = url;
+    values[s_descr]  = descr;
+    values[s_handler]  = handler;
+
+    if ( !streamStorage->insertRecord(106, values, errorMsg) )
+    {
+       cout << TARGET" download create error: " << errorMsg << endl;
+       return false;
+    }
+       else return true;
+}
+
+bool Downloader::updateStreamItemFilename(QString oldFile, QString newFile)
+{    
+    QString errorMsg = "";
+    
+    bool found = false;
+    
+    streamStorage->resetRecordList();
+
+    ValueList values(5);
+    while ( !found && streamStorage->getNextRecord(values) )
+    {
+      if ( values.count() == 5 && values[s_folder] == "downloads" && values[s_url] == oldFile )
+        found = true;
+    }
+
+    if (!found)
+    {
+      cout << TARGET" error: download stream item url " << oldFile << " not found" << endl;
+      return false;
+    }
+    
+    ValueList ovalues = values;
+
+    values[s_url] = newFile;
+    
+    if ( !streamStorage->updateRecord(106, ovalues, values, errorMsg) )
+    {
+       cout << TARGET" download update error: " << errorMsg << endl;
+       return false;
+    }
+       else return true;
+}
+
+bool Downloader::deleteStreamItem(QString name, QString url, QString descr, QString handler)
+{
+    QString errorMsg;
+    ValueList values(5);
+    values[s_folder] = "downloads";
+    values[s_name]   = name;
+    values[s_url]    = url;
+    values[s_descr]  = descr;
+    values[s_handler]  = handler;
+
+    if ( !streamStorage->removeRecord(106, values, errorMsg) )
+    {
+       cout << TARGET" download delete error: " << errorMsg << endl;
+       return false;
+    }
+       else return true;
+    
+}
+
+void Downloader::slotRecordRemoved(ChangedRecord* rec)
+{
+  if (rec->ident == 106) return;
+
+  // delete file if: folder=downloads
+  if ( rec->oldValues[0] == "downloads" )
+  {
+     //stopRecording(rec->values[1]);
+     QFile(rec->oldValues[2]).remove();
+  }
+}
+
+
+
 //----------------------------------------------------------------------------
 // using cache is nice when working playlists, required when working rss/rdf feeds
 
@@ -278,12 +414,12 @@ bool Cache::copyCacheFile(QString source, QString dest)
 
   if ( !src.open(IO_ReadOnly) )
   {
-    cout << "cannot open " << source << endl;
+    cout << TARGET" error: cannot open " << source << endl;
     return false;
   }
   if ( !dst.open(IO_WriteOnly) )
   {
-    cout << "cannot open " << dest << endl;
+    cout << TARGET" error: cannot open " << dest << endl;
     return false;
   }
 
@@ -736,7 +872,7 @@ bool Requester::fetchData(QString url, QString fileName, QString& errorMsg)
     {
        delete fetchBuffer;
        fetchBuffer = 0;
-       cout << "mythstream error: " << errorMsg << endl;
+       cout << TARGET" error: " << errorMsg << endl;
        return false;
     }
       
@@ -840,7 +976,7 @@ void Requester::slotResponseHeaderReceived(const QHttpXResponseHeader& response)
        status = abort;
        http->abort();
        fetchBuffer = 0;
-       cout << "MythStream error: " << errorMsg << endl;
+       cout << TARGET" error: " << errorMsg << endl;
        return;
     }
       
@@ -896,6 +1032,8 @@ void Requester::slotReadyRead(const QHttpXResponseHeader&)
    // count the number of complete url's in this chunk
    // add to total, do some math and decide to continue
 
+   // can be wrong, but code below is useless since default parser delegation?
+   
     // check for protocol://server.domain
     int index = 0;
     QRegExp expr;
@@ -1379,7 +1517,7 @@ void StreamHarvester::processExited()
      return;
    }
    
-   externalParserStart(currentUrl, currentHandler);
+   externalParserStart(currentUrl, currentHandler, currentName);
 }
 
 void StreamHarvester::slotFetchProgress(int read, int) // data read by Requester
@@ -1454,7 +1592,7 @@ void StreamHarvester::parserExited(int matchCount)
 
 // outsourcing the parser factory...
 
-void StreamHarvester::externalParserStart(QString srcUrl, QString parser)
+void StreamHarvester::externalParserStart(QString srcUrl, QString parser, QString srcName)
 {
     if ( eParser ) return;
 
@@ -1480,8 +1618,6 @@ void StreamHarvester::externalParserStart(QString srcUrl, QString parser)
     
     getParser(parser, ppath);
     
-    cout << "resulting parser: " << parser << " workdir: " << ppath << endl;
-
     sourceUrl = srcUrl;
     
     urlCount = 0;
@@ -1496,21 +1632,35 @@ void StreamHarvester::externalParserStart(QString srcUrl, QString parser)
     //                        |QProcess::Stderr
     //                        |QProcess::DupStderr
                             );
-                            
+    // The source Url must be quoted to avoid the shell
+    // interpreting special url characters like &,, *, ...
+    // The source Url can therefore not contain quotes itself.
+    if ( sourceUrl.find('"') != -1 || sourceUrl.find('\\') != -1)
+    {
+      cerr << "Mythstream: insecure sourceURL: " << sourceUrl << endl;
+      sourceUrl = "";
+    }
+                  
+    srcName = srcName.replace('"', "'");
+
+    while (srcName.right(1) == '\\')
+      srcName.truncate( srcName.length() );
  
 #if PARSER == PERL
     // execute perl script
     eParser->addArgument( "perl" );
     eParser->addArgument( parser );
     eParser->addArgument( "list.xml" );
-    eParser->addArgument( sourceUrl );  // change
+    eParser->addArgument( "\"" + sourceUrl + "\"" );  // change
+    eParser->addArgument( "\"" + srcName + "\"" );  // change
     //cout << "PERL PARSER" << endl;
 
 #elif PARSER == EXEC
     // executable binary
     eParser->addArgument( parser );
     eParser->addArgument( "list.xml" );
-    eParser->addArgument( sourceUrl );  // change
+    eParser->addArgument( "\"" + sourceUrl + "\"");  // change
+    eParser->addArgument( "\"" + srcName + "\"");  // change
     //cout << "EXEC PARSER" << endl;
 
 #elif PARSER == SHELL
@@ -1520,17 +1670,10 @@ void StreamHarvester::externalParserStart(QString srcUrl, QString parser)
     // requires parser files to be executable!
     // parser and sourceUrl are external
     // trust parser: assume the perl script defines that value
-    // do not trust sourceUrl provided by webserver
-    //    sourceUrl is quoted, cannot contain quotes
-    if ( sourceUrl.find('"') != -1 )
-    {
-      cerr << "MythStream: insecure sourceURl: " << sourceUrl << endl;
-      sourceUrl = "";
-    }
     
     eParser->addArgument( "sh" );
     eParser->addArgument( "-c" );
-    eParser->addArgument( parser + " list.xml \"" + sourceUrl + "\"" );
+    eParser->addArgument( parser + " list.xml \"" + sourceUrl + "\"" + srcName + "\"" );
     //cout << "SHELL PARSER" << endl;
     
 #endif
