@@ -39,7 +39,7 @@
 using namespace std;
 
 /*
-  labels in file:
+  labels in file and web storage:
   [item]          record header
   [emptystring]   represents empty string
   [rmvd]          removed item header
@@ -208,7 +208,7 @@ bool GenStorage::loadList(int, QString& error)
 {
     if ( accessRequestType != idle )
     {
-      error = "storage is busy";
+      error = "storage " + accessName + " is busy or unavailable";
       return false;
     }
 
@@ -838,7 +838,7 @@ bool DatabaseStorage::insertRecord(int ident, ValueList& values, QString& error)
 {
     if ( !GenStorage::insertRecord(ident, values, error) )
       return false;
-
+cout << "db storage insert: " << values[1] << endl;
     changedRecord->error = !insertDbRecord();
     if ( changedRecord->error )
       accessLastError = "cannot write to storage";
@@ -978,6 +978,13 @@ bool DatabaseStorage::saveListToDb(RecordList& list)
     return result;
 }
 
+// Proper escaping please...
+QString escapeValue(QString val)
+{
+   val = val.replace("'","''");
+   return val;
+}
+
 bool DatabaseStorage::insertDbRecord()
 {
     QString queryString;
@@ -997,7 +1004,7 @@ bool DatabaseStorage::insertDbRecord()
     for ( int i = 0; i < cnt; i++ )
     {
       if (i > 0) queryString += ", ";
-      queryString += "'" + changedRecord->values[i] + "'";
+      queryString += "'" + escapeValue( changedRecord->values[i] ) + "'";
       record->values.append( changedRecord->values[i] );
     }
 
@@ -1028,14 +1035,14 @@ bool DatabaseStorage::updateDbRecord()
     for ( int i = 0; i < cnt; i++ )
     {
       if (i>0) { temp1 += ", "; temp2 += " and "; }
-      temp1 += accessDbFields[i] + "='" + changedRecord->values[i] + "'";
-      temp2 += accessDbFields[i] + "='" + changedRecord->oldValues[i] + "'";
+      temp1 += accessDbFields[i] + "='" + escapeValue( changedRecord->values[i] ) + "'";
+      temp2 += accessDbFields[i] + "='" + escapeValue( changedRecord->oldValues[i] ) + "'";
       if (rec) rec->values[i] = changedRecord->values[i];
     }
 
     queryString += temp1 + " where " + temp2;
     MSqlQuery query(MSqlQuery::InitCon());
-
+    
     return query.exec(queryString);
 }
 
@@ -1049,7 +1056,7 @@ bool DatabaseStorage::removeDbRecord()
     for ( int i = 0; i < cnt; i++ )
     {
       if (i>0) queryString += " and ";
-      queryString += accessDbFields[i] + "='" + changedRecord->values[i] + "'";
+      queryString += accessDbFields[i] + "='" + escapeValue( changedRecord->values[i] ) + "'";
     }
 
     MSqlQuery query(MSqlQuery::InitCon());
@@ -1413,9 +1420,12 @@ void WebStorage::parseWebList(QString& data)
           else if (stage > 1)
        {
            if (*i)
-             record->values.append(*i);
-           else
-             record->values.append("");
+           {
+					if (*i != "[emptystring]")
+						record->values.append(*i);
+					else
+						record->values.append("");
+           }
        }
     }
 
