@@ -50,7 +50,7 @@ ViewerWindow::ViewerWindow(QWidget *parent, const char *name, WFlags f)
   hide();
   setCaption(name);
   parentWidget = parent;
-  
+
   QVBoxLayout *vbox = new QVBoxLayout(this, (int)(10));
 
   QFont fo("monospace");
@@ -159,6 +159,7 @@ StreamBrowser::StreamBrowser(QWidget *parent, QWidget *focusProxy, StreamStorage
     video = false;
     m_dumpwindow = 0;
     markedStorage = NULL;
+    playListActive = false;
     
     setItemDisplayFolded( -1, false);
     setFolderDisplayFolded( -1, false );
@@ -169,9 +170,11 @@ StreamBrowser::StreamBrowser(QWidget *parent, QWidget *focusProxy, StreamStorage
     streamStatus    = new StreamStatus(this);
     streamHarvester = new StreamHarvester();
 
-    QString downloadPath = getenv("HOME");
-    downloadPath += "/."SUBPATH"/downloads";
-    downloader = new Downloader( downloadPath );
+    oldStatus = StreamStatus::idle;
+    
+    QString downloadPath = ""; //getenv("HOME");
+    downloadPath += "."SUBPATH"/downloads";
+    downloader = new Downloader( downloadPath, storage );
     
     connect( streamStatus,
              SIGNAL(statusChange()),
@@ -300,11 +303,14 @@ void StreamBrowser::destroyedDumpWindow()
 bool StreamBrowser::eventFilter( QObject *obj, QEvent *ev)
 {
   // events for VideoContainer, already handled by mplayer
-
+  //cout << "event passed: " << ev->type() << endl;
+    
   if (obj && ev->type() == QEvent::KeyPress )
   {
+    
     // pass key if not handled by mplayer (and how would I know it isn't?)
     QKeyEvent *kev = static_cast<QKeyEvent*>(ev);
+
     if ( kev->key() != Key_P )
       qApp->sendEvent(focusProxy, ev);
     return true;
@@ -323,16 +329,13 @@ bool StreamBrowser::eventFilter( QObject *obj, QEvent *ev)
 }
 
 
-void StreamBrowser::reportEvent(const QString& eventInfo)
+void StreamBrowser::reportEvent(const QString& eventMsg, QString eventInfo)
 {
-   userMessage = eventInfo;
-   emit eventUserMessage();
+   userMessage = eventMsg;
+   userInfo    = eventInfo;
+   emit eventUserMessage(userMessage, userInfo);
 }
 
-QString& StreamBrowser::getUserMessage()
-{
-   return userMessage;
-}
 
 void StreamBrowser::setVideoRect(QRect& area)
 {
@@ -366,6 +369,11 @@ void StreamBrowser::showVideo()
    if (video) videoContainer->show();
 }
 
+bool StreamBrowser::videoShown()
+{
+   return (video && videoContainer->isVisible());
+}
+
 void StreamBrowser::hideViewer()
 {
    viewerWindow->hide();
@@ -603,7 +611,7 @@ void StreamBrowser::loadDumpWindow()
 void StreamBrowser::streamStatusChanged()
 {
     loadDumpWindow();
-    
+
     if ( video != streamStatus->isVideo() )
     {
       video = streamStatus->isVideo();
@@ -617,13 +625,21 @@ void StreamBrowser::streamStatusChanged()
 
     emit eventValuesUpdated( streamstatus );
 
-    if ( newStatus == StreamStatus::playing )
+    if ( newStatus == StreamStatus::playing && oldStatus != StreamStatus::paused )
     {
       streamStatus->issueCommand(StreamStatus::VolumeDn);
       streamStatus->issueCommand(StreamStatus::VolumeUp);
+      if (playListActive)
+         QTimer::singleShot( 5000, this, SLOT(timerSaysFullScreen()) );
     }
           
     samplerActive(newStatus == StreamStatus::playing);
+
+    if ( newStatus == StreamStatus::stopped )
+      playNextMarkedStream();
+
+    oldStatus = newStatus;
+
 }
 
 unsigned int StreamBrowser::getStreamCustomParameterCount()
@@ -647,7 +663,8 @@ QString StreamBrowser::getStreamCustomParameter(unsigned int index, QString& are
 QString StreamBrowser::getStreamParameter(Parameter parameter)
 {
     QString text = "";
-
+    int secs;
+    
     switch ( parameter )
     {
       case streamName: text = streamStatus->getStreamName();
@@ -679,6 +696,11 @@ QString StreamBrowser::getStreamParameter(Parameter parameter)
       case streamCache: text = streamStatus->getStreamInfo(StreamStatus::StreamCache);
       break;
       case streamTime: text = streamStatus->getStreamInfo(StreamStatus::StreamTime);
+           secs = text.toInt();
+           if (secs != 0)
+           {
+					text.sprintf("%02d:%02d", secs / 60, secs % 60);
+           }
       break;
       case streamStability: text = streamStatus->getStreamInfo(StreamStatus::StreamStability);
       break;
@@ -699,8 +721,10 @@ void StreamBrowser::streamPollEvent()
 {
    // StreamStatus pollSignal emitted every second if stream is not idle
 
-    emit eventValuesUpdated( streamplay );
-
+   if ( ! streamStatus->isFullScreen() )
+     emit eventValuesUpdated( streamplay );
+   else
+     emit eventValuesUpdated( fsstreamplay );
 }
 
 void StreamBrowser::streamUndetected()
@@ -720,6 +744,20 @@ void StreamBrowser::streamUndetected()
    }
 }
 
+void StreamBrowser::playNextMarkedStream()
+{
+    StreamFolder *folder = itemTree->getStreamFolder();
+      
+	if (folder && folder->nextMarkedObject())
+	{
+		folder->cursor = folder->getIndex();
+		emit eventValuesUpdated( itemlist );
+		playListActive = true;
+		handlePressedKey(selectitem);
+	}
+	   else playListActive = false;
+}
+
 void StreamBrowser::fillHarvestList(bool filterStreams)
 {
     StreamItem* item;
@@ -759,13 +797,18 @@ void StreamBrowser::fillHarvestList(bool filterStreams)
         item = new StreamItem(folder, url->name, url->url, url->descr, url->handler);
         item->handler = url->handler;
         item->meta = url->meta;
-        if (item->handler == "STREAM_DL" )
+
+        // see if exists
+        // STREAM_DL: url must match
+        // FUZZY_DL : url or name of stream item in download folder must match
+
+        if (item->handler == "STREAM_DL" || item->handler == "FUZZY_DL" )
         {
           item->setAction(download);
           prefix = "D"; // download icon
           Downloader::Availability availability;
           bool dummy;
-          downloader->checkAvailable(url->url, availability, dummy, false);
+          downloader->checkAvailable( url->url, availability, dummy, false, url->name, (item->handler == "FUZZY_DL") );
 
           switch (availability)
           {
@@ -864,7 +907,7 @@ void StreamBrowser::slotDownloadFinished(QString url, bool error, QString errorM
       emit eventValuesUpdated( itemlist );
 
    if (error)
-      reportEvent(errorMessage);
+      reportEvent(errorMessage, "");
 
 }
 
@@ -1019,7 +1062,7 @@ void StreamBrowser::storeMarkedStreamsPrepare(QString storageName)
    
    if (!loaded)
    {
-      reportEvent( "error: " + markedStorage->getLastError() );
+      reportEvent( "error: " + markedStorage->getLastError(), "" );
       delete(markedStorage);
       streamCopyList.clear();
       markedStorage = NULL;
@@ -1073,7 +1116,7 @@ bool StreamBrowser::storeMarkedStreamsInsertNext(bool first)
       }
    }
       
-   if (!inserted) // no pensing recordinserted signals...
+   if (!inserted) // no pending recordinserted signals...
    {
       streamCopyIndex = -1;
       streamCopyList.clear();
@@ -1242,7 +1285,18 @@ void StreamBrowser::setInfoMode(bool goInfoMode)
                 prefix = "I";
                 inlineText += ": " + *metaparts.at(2);
               }
-    
+/*
+        new viewer "url" could be used to provide feedback (e.g. mark nonfunctional)
+        
+        Example parser implementation:
+
+        $meta = $doc->createElement('meta');
+        $meta->appendChild( newNode('name'   , 'action: mark as nonfunctional') );
+        $meta->appendChild( newNode('content', "wwmp/wwmp_act|marknf|$stationid") );
+        $meta->appendChild( newNode('viewer' , 'url') );
+        $item->appendChild( $meta );
+
+*/
               item = new StreamItem( folder1, inlineText, "", "meta data", "" );
               item->meta.append(*metaparts.at(2)); // dump content in first position
               item->setPrefix(prefix);
@@ -1329,7 +1383,7 @@ to save multiple harvested items to storage";
         
         if (!loaded)
         {
-           reportEvent("cannot load storage repository");
+           reportEvent("cannot load storage repository", "");
            return;
         }
         
@@ -1541,37 +1595,118 @@ url itself, the player will stay in harvester mode";
       itemTreeLoaded = streams; 
 }
 
+void StreamBrowser::updateStreamItem(StreamItem* item, QString oldUrl)
+{
+    QString error;
+
+    ValueList values(5);
+    values[0] = item->getFolderName();
+    values[1] = item->getName();
+    values[2] = item->url;
+    values[3] = item->descr;
+    values[4] = item->handler;
+
+    ValueList oldValues(5);
+    oldValues[0] = item->getFolderName();
+    oldValues[1] = item->getName();
+    oldValues[2] = oldUrl;
+    oldValues[3] = item->descr;
+    oldValues[4] = item->handler;
+
+  //if ( item && !
+  streamStorage->updateRecord(ident_streambrowser, oldValues, values, error);
+  //  cout << TARGET"storage error: " << error << endl;
+}
+
+QString urlenc(QString line)
+{
+   QUrl::encode(line);
+   return line;
+}
 
 // start streamer
 void StreamBrowser::initStream()
 {
+   QString line = "";
+   QString oldUrl = "";
+   QRegExp expr;
+   bool hasParameter = false;
+   bool cancelled = false;
+
+   int index = 0;
+   QString url;
+   
    StreamFolder *folder = itemTree->getStreamFolder();
    if (folder)
    {
       StreamItem* pendingStation = folder->getStreamItem();
       if (pendingStation)
       {
-          streamStatus->initStream( 
-                             pendingStation->url,
-                             pendingStation->getName(),
-                             pendingStation->descr,
-                             pendingStation->handler 
-          );
+          oldUrl = url = pendingStation->url;
+
+          // check for <:name::value:> in url
+          expr.setPattern("<:(.*)::(.*):>");
+          expr.setMinimal(true);
+
+          // ask user to edit/confirm value
+          while ( (index = expr.search( url, index )) && index > -1 && !cancelled )
+          {
+            hasParameter = true;
+            line = expr.cap(2);
+            emit eventEditString(line, expr.cap(1), cancelled);
+            url.replace(index, expr.matchedLength(), "<:" + expr.cap(1) + "::" + line + ":>");
+            index++;
+          }
+
+          if (cancelled) return;
+          
+          // update url in stream storage, replace <:name::value:> in url with value
+          if (hasParameter)
+          {
+             pendingStation->url = url;
+             if (oldUrl != url) updateStreamItem(pendingStation, oldUrl);  // ignore (readonly) write error
+             index = 0;
+             while ( (index = expr.search( url, index )) && index > -1 )
+               url.replace(index, expr.matchedLength(), urlenc(expr.cap(2)));
+             //cout << "new url is: " << url << endl;
+          }
+
+          playListActive = pendingStation->marked;
           
           if (pendingStation->handler != "")  // parser change
           {
              streamStatus->stopStream();
-             streamUndetected();
-      
-      /*
-      emit eventHarvesterBusy( true, "fetching data" );
+             //streamUndetected();
+				streamHarvester->setTagOnCurrent( folder->getIndex() );
 
-      streamHarvester->fetchData( streamStatus->getStreamUrl(),
-                                  streamStatus->getStreamName(),
-                                  streamStatus->getStreamDescr(),
-                                  streamStatus->getStreamHandler() );
-      */       
+				emit eventHarvesterBusy( true, "fetching data" );
+				streamHarvester->fetchData(
+											url,
+											pendingStation->getName(),
+											pendingStation->descr,
+											pendingStation->handler
+													);
+
+				/*
+				emit eventHarvesterBusy( true, "fetching data" );
+
+				streamHarvester->fetchData( streamStatus->getStreamUrl(),
+													streamStatus->getStreamName(),
+													streamStatus->getStreamDescr(),
+													streamStatus->getStreamHandler() );
+				*/
           }
+            else
+          {
+          
+          streamStatus->initStream( 
+                             url,
+                             pendingStation->getName(),
+                             pendingStation->descr,
+                             pendingStation->handler 
+          );
+          }
+            
       }
    }
 }
@@ -1598,14 +1733,15 @@ void StreamBrowser::startDownload()
       StreamItem* pendingStation = folder->getStreamItem();
       if (pendingStation)
       {
-         bool error, canplaynow;
+         bool error, canplaynow, itemcreated;
          QString reason;
          QString file;
          
-         file = downloader->download(pendingStation->url, canplaynow, error, reason);
+         file = downloader->download(pendingStation->url, pendingStation->getName(), canplaynow, itemcreated, 
+                 error, reason, ( pendingStation->handler == "FUZZY_DL") );
          
          if (error)
-           reportEvent(reason);
+           reportEvent(reason, "");
          else
            if (canplaynow)
              streamStatus->initStream( file,
@@ -1615,8 +1751,15 @@ void StreamBrowser::startDownload()
            else
            {
              pendingStation->setPrefix("R"); // downloading
+
              emit eventValuesUpdated ( itemlist );
-             reportEvent("A file download has started. Select the stream item again to play the (partially) downloaded file.");
+
+             if (!itemcreated)
+               reason = "Could not create stream item in downloads folder, downloading anyway";
+             else
+               reason = "";
+             
+             reportEvent("A file download has started. Select the stream item again to play the (partially) downloaded file.", reason);
            }
       }
    }
@@ -1728,7 +1871,7 @@ void StreamBrowser::initRecording(QString url, QString name, QString /*descr*/,
    recName = recorderManager->recordNow(url, name, 3600, errorMsg);
 
    if (recName == "")
-     reportEvent(errorMsg);
+     reportEvent(errorMsg, "");
 }
 
 void StreamBrowser::slotRecordingStopped(QString recName, RecordStopReason reason)
@@ -1750,16 +1893,16 @@ void StreamBrowser::slotRecordingStopped(QString recName, RecordStopReason reaso
    switch (reason)
    {
      case command:
-        reportEvent("Finished recording " + recName);
+        reportEvent("Finished recording " + recName, "");
      break;
      case process:
-        reportEvent("Could not start recorder for " + recName);
+        reportEvent("Could not start recorder for " + recName, "");
      break;
      case file:
-        reportEvent("Could not open storage for recording " + recName); // not used
+        reportEvent("Could not open storage for recording " + recName, ""); // not used
      break;
      case recorder:
-        reportEvent("The recorder exited. Try to press record when playing the stream");
+        reportEvent("The recorder exited. Try to press record when playing the stream", "");
      break;
    }
 }
@@ -1778,7 +1921,7 @@ void StreamBrowser::slotRecordingStarted(QString recName)
       }
    }
 
-   reportEvent("Started recording " + recName);
+   reportEvent("Started recording " + recName, "");
 }
 
 void StreamBrowser::slotScheduleEvent(QString recName, QString errorMessage, bool /*isScheduled*/)
@@ -1811,7 +1954,7 @@ void StreamBrowser::slotScheduleEvent(QString recName, QString errorMessage, boo
    }
 
    if (errorMessage != "") // old recordings
-     reportEvent(errorMessage + " " + recName);
+     reportEvent(errorMessage + " " + recName, "");
 
 }
 
@@ -1898,7 +2041,7 @@ void StreamBrowser::slotStorageEvent(int ident, int eventType, bool error)
           {
               cerr << TARGET": cannot read from storage"
               << markedStorage->getStorageDescription() << endl;
-              reportEvent(errorMsg);
+              reportEvent(errorMsg, "");
               delete( markedStorage );
           }
     default:
@@ -1911,6 +2054,11 @@ void StreamBrowser::markedStreamsInsertReady()  //sent by timer after last inser
    storeMarkedStreamsInsertNext(false);
 }
 
+void StreamBrowser::timerSaysFullScreen()  //sent by timer after last insert
+{
+   streamStatus->toggleFullScreen(true);
+}
+
 void StreamBrowser::slotMarkedRecordInserted(ChangedRecord* /*cr*/) 
 {
    storeMarkedStreamsInsertNext(false);
@@ -1987,7 +2135,7 @@ void StreamBrowser::slotListLoaded()
     emit eventValuesUpdated( folderlist );
     
     if (error)
-      reportEvent("Errors in stream storage. Run from command line to see details");
+      reportEvent("Errors in stream storage. Run from command line to see details", "");
       
 }
 
@@ -2019,9 +2167,9 @@ void StreamBrowser::slotRecordInserted(ChangedRecord* rec)
   emit eventValuesUpdated( folderlist );
 
   if ( rec->oldValues[1] != rec->values[1] )
-    reportEvent( "Saved \"" + rec->oldValues[1] + "\" as \"" + rec->values[1] + "\"");
+    reportEvent( "Saved \"" + rec->oldValues[1] + "\" as \"" + rec->values[1] + "\"", "");
   else
-    reportEvent( "Item \"" + rec->values[1] + "\" saved");
+    reportEvent( "Item \"" + rec->values[1] + "\" saved", "");
 }
 
 void StreamBrowser::slotRecordUpdated(ChangedRecord* rec)
@@ -2066,7 +2214,7 @@ void StreamBrowser::slotRecordUpdated(ChangedRecord* rec)
 
   emit eventValuesUpdated( folderlist );
 
-  reportEvent( "Item \"" + rec->oldValues[1] + "\" updated");
+  reportEvent( "Item \"" + rec->oldValues[1] + "\" updated", "");
 }
 
 void StreamBrowser::slotRecordRemoved(ChangedRecord* rec)
@@ -2099,9 +2247,9 @@ void StreamBrowser::slotRecordRemoved(ChangedRecord* rec)
   emit eventValuesUpdated( folderlist );
 
   if (!fileDeleted)
-    reportEvent( "Item \"" + rec->values[1] + "\" removed");
+    reportEvent( "Item \"" + rec->values[1] + "\" removed", "");
   else
-    reportEvent( "Item + file \"" + rec->values[1] + "\" removed");
+    reportEvent( "Item + file \"" + rec->values[1] + "\" removed", "");
 }
 
 void StreamBrowser::storeStationInFolder(QString folderName)
@@ -2136,7 +2284,7 @@ void StreamBrowser::storeLinkInFolder(QString folderName)
 
   if ( !item )
   {
-    reportEvent( "No link to save");
+    reportEvent( "No link to save", "");
     return;
   }
 
@@ -2167,7 +2315,7 @@ void StreamBrowser::storeItemInFolder(QString folderName, QString name, QString
 
   if ( url == "" )
   {
-     reportEvent( "No url specified" );
+     reportEvent( "No url specified", "");
      return;
   }
 
@@ -2183,7 +2331,7 @@ void StreamBrowser::storeItemInFolder(QString folderName, QString name, QString
   bool res = streamStorage->insertRecord(101, values, errorMsg);
   if ( !res)
   {
-      reportEvent( errorMsg );
+      reportEvent( errorMsg, "");
       return;
   }
 
@@ -2201,9 +2349,13 @@ bool StreamBrowser::handlePressedKey(int key)
     else if (key == nextfolder)
         folderNext();
     else if (key == previtem)
-        itemPrev();
+        itemPrev(1);
     else if (key == nextitem)
-        itemNext();
+        itemNext(1);
+    else if (key == jmpprevitem)
+        itemPrev(6);
+    else if (key == jmpnextitem)
+        itemNext(6);
     else if (key == selectitem)
           checkFolderCommand();
     else if (key == mark)
@@ -2240,7 +2392,7 @@ bool StreamBrowser::handlePressedKey(int key)
     else if (key == rewindstream)
         streamStatus->issueCommand(StreamStatus::Rewind);
     else if (key == fullscreen) // fullscreen while video plays, debugwindow when no video
-        streamStatus->toggleFullScreen();
+        streamStatus->toggleFullScreen(false);
     else if (key == dump)
         toggleDumpWindow();
     else if (key == escape)
@@ -2267,13 +2419,16 @@ void StreamBrowser::folderPrev()
    emit eventValuesUpdated( folderlist );  // implies itemlist update
 }
 
-void StreamBrowser::itemPrev()
+void StreamBrowser::itemPrev(uint cnt)
 {
     StreamFolder *folder = itemTree->getStreamFolder();
     if (folder)
     {
-      folder->prevObject(m_itemdisplayfolded);
-      folder->cursor--;
+      for (uint i = 0; i < cnt; i++)
+      {
+        folder->prevObject(m_itemdisplayfolded);
+        folder->cursor--;
+      }
     }
     emit eventValuesUpdated( itemlist );
 }
@@ -2285,13 +2440,16 @@ void StreamBrowser::folderNext()
     emit eventValuesUpdated( folderlist );  // implies itemlist update
 }
 
-void StreamBrowser::itemNext()
+void StreamBrowser::itemNext(uint cnt)
 {
     StreamFolder *folder = itemTree->getStreamFolder();
     if (folder)
     {
-      folder->nextObject(m_itemdisplayfolded);
-      folder->cursor++;
+      for (uint i = 0; i < cnt; i++)
+      {
+        folder->nextObject(m_itemdisplayfolded);
+        folder->cursor++;
+      }
     }
     emit eventValuesUpdated( itemlist );
 }
@@ -2332,7 +2490,7 @@ void StreamBrowser::handleExternalURI( QString uri )
 
       if (!file.exists() || !file.open(IO_ReadOnly))
       {
-         reportEvent("cannot open " + uri);
+         reportEvent("cannot open " + uri, "");
          return;
       }
 
