@@ -35,20 +35,26 @@
 using namespace std;
 /*
 
+  StorageGroup has been renamed to MStorageGroup: it conflicted with new mythtv class StorageGroup
+  
   Needs cleanup
-
-  StreamTunedConfig: main config window with "stream" and "storage" tabs
+  
+Window hierarchy
+  StreamTunedConfig: main config window with "stream" and "storage" tabs  //StreamTuned only
+  gContext->GetMainWindow() // MythTV only
     StreamConfig: stream configuration handler (stream tab)
       StorageGroup: StreamConfig goupBox with storage controls
       StreamEditGroup: StreamConfig groupBox with stream property edit boxes
    (StorageConfig: storage configuration handler (storage tab) see storageconfig.h)
 
+  StreamConfig and StorageConfig are called from plugins main.cpp
+
 */
 
 
 // ------------------- Storage Select group box --------------------
 
-StorageGroup::StorageGroup(char* title, QWidget* parent, ReposStorage *storageStorage,
+MStorageGroup::MStorageGroup(char* title, QWidget* parent, ReposStorage *storageStorage,
                            StreamStorage *streamStorage) : QVGroupBox(title, parent)
 {
     myParent = parent;
@@ -129,7 +135,7 @@ StorageGroup::StorageGroup(char* title, QWidget* parent, ReposStorage *storageSt
 
 }
 
-void StorageGroup::fillStorageBox()
+void MStorageGroup::fillStorageBox()
 {
     ValueList values;
 
@@ -145,7 +151,7 @@ void StorageGroup::fillStorageBox()
 
 }
 
-void StorageGroup::buildGroup()
+void MStorageGroup::buildGroup()
 {
     QFontMetrics metric( font() );
     int maxHeight = metric.height() + 2;
@@ -178,12 +184,12 @@ void StorageGroup::buildGroup()
 
 }
 
-StorageGroup::~StorageGroup()
+MStorageGroup::~MStorageGroup()
 {
 
 }
 
-void StorageGroup::synchronized(bool sync)
+void MStorageGroup::synchronized(bool sync)
 {
   if (!sync)
   {
@@ -201,7 +207,7 @@ void StorageGroup::synchronized(bool sync)
   }
 }
 
-void StorageGroup::reportMessage( const QString& message, bool bad)
+void MStorageGroup::reportMessage( const QString& message, bool bad)
 {
    StreamConfig *streamConfig = dynamic_cast<StreamConfig*>(myParent);
 
@@ -210,7 +216,7 @@ void StorageGroup::reportMessage( const QString& message, bool bad)
 }
 
 
-void StorageGroup::slotBoxActivated(const QString& selectedText)
+void MStorageGroup::slotBoxActivated(const QString& selectedText)
 {
     if ( selectedText == streams->getStorageName() ) return;
 
@@ -233,21 +239,21 @@ void StorageGroup::slotBoxActivated(const QString& selectedText)
     }
 }
 
-void StorageGroup::slotLoadClicked()
+void MStorageGroup::slotLoadClicked()
 {
    QString error;
    if ( !streams->loadList(ident_storagegroup, error) )
      reportMessage(error, true);
 }
 
-void StorageGroup::slotOverwriteClicked()
+void MStorageGroup::slotOverwriteClicked()
 {
    QString error;
    if ( !streams->storeList(ident_storagegroup, error) )
      reportMessage(error, true);
 }
 
-void StorageGroup::slotStorageEvent(int ident, int eventType, bool error)
+void MStorageGroup::slotStorageEvent(int ident, int eventType, bool error)
 {
   if ( error )
   {
@@ -266,14 +272,14 @@ void StorageGroup::slotStorageEvent(int ident, int eventType, bool error)
   }
 }
 
-void StorageGroup::slotRecordInserted(ChangedRecord *rec)
+void MStorageGroup::slotRecordInserted(ChangedRecord *rec)
 {
   if ( rec->error ) return;
 
   storageBox->insertItem( rec->values[r_name] );
 }
 
-void StorageGroup::slotRecordUpdated(ChangedRecord *rec)
+void MStorageGroup::slotRecordUpdated(ChangedRecord *rec)
 {
   if ( rec->error ) return;
 
@@ -284,7 +290,7 @@ void StorageGroup::slotRecordUpdated(ChangedRecord *rec)
     }
 }
 
-void StorageGroup::slotRecordRemoved(ChangedRecord *rec)
+void MStorageGroup::slotRecordRemoved(ChangedRecord *rec)
 {
   if ( rec->error ) return;
 
@@ -304,7 +310,7 @@ void StorageGroup::slotRecordRemoved(ChangedRecord *rec)
   // last storage deleted
 }
 
-void StorageGroup::slotStreamStorageEvent(int ident, int eventType, bool error)
+void MStorageGroup::slotStreamStorageEvent(int ident, int eventType, bool error)
 {
 
   if ( error )
@@ -703,7 +709,7 @@ void StreamConfig::setupView()
     streamEditGroup = new StreamEditGroup("Edit webcast", this);
     detailvbox->addWidget(streamEditGroup);
 
-    storageGroup = new StorageGroup("Storage handling", this, storageStorage, streamStorage);
+    storageGroup = new MStorageGroup("Storage handling", this, storageStorage, streamStorage);
     detailvbox->addWidget(storageGroup);
 
     connect(m_leftView, SIGNAL(selectionChanged(QListViewItem*)),
