@@ -23,6 +23,7 @@
 
 #include <iostream>
 
+#include <qapplication.h>
 #include <qnetwork.h>
 #include <qdatetime.h>
 #include <qpainter.h>
@@ -38,10 +39,14 @@
 #include <qurl.h>
 #include <math.h>
 
+
 #include "streamstatus.h"
 
 void myX11MapWindow (uint id);
 void myX11UnmapWindow (uint id);
+uint myX11GrabKeyboard(uint id);
+uint myX11UngrabKeyboard();
+
 //void listenXEvents(uint windowid, XEvent *event);
 
 using namespace std;
@@ -49,21 +54,133 @@ using namespace std;
 // ----------------------------------------------------------------------
 
 
+VideoArea::VideoArea(QWidget *parent, const char *name, WFlags f)
+               : QWidget(parent, name, f)
+{
+   myParent = parent;
+   //setFocusPolicy(QWidget::StrongFocus);
+}
+
+void VideoArea::focusInEvent ( QFocusEvent *ev )
+{
+   QWidget::focusInEvent(ev);
+}
+
+void VideoArea::focusOutEvent ( QFocusEvent *ev )
+{
+   QWidget::focusOutEvent(ev);
+}
+
 VideoContainer::VideoContainer(QWidget *parent, const char *name, WFlags f)
                : QWidget(parent, name, f)
 {
     hide();
+    
+    par = parent;
     setCaption(name);
+    setBackgroundMode(NoBackground);
+
+    vidArea = new VideoArea(this, "videoArea", 0);
+    setBackgroundColor( Qt::black );
+    vidArea->show();
+
+    lastZoom = false;
+    keyboardGrabbed = false;
+
+    hide();
 }
 
 QString VideoContainer::getVidWindowId()
 {
-    return QString::number(winId(), 10);
+    return QString::number(vidArea->winId(), 10);
 }
 
 int VideoContainer::getMyWindowId()
 {
-    return winId();
+    return vidArea->winId();
+}
+
+void VideoContainer::goFullscreen(double aspectRatio, bool zoom)
+{
+    setGeometry( screenRect );
+    vidArea->setGeometry( calcVideoRectFromAspect(aspectRatio, screenRect, zoom) );
+    lastZoom = zoom;
+    
+    if (!keyboardGrabbed)
+      keyboardGrabbed = ( myX11GrabKeyboard(getMyWindowId()) == 0 );
+}
+
+void VideoContainer::showEvent( QShowEvent * )
+{
+   keyboardGrabbed = false;
+}
+
+void VideoContainer::goPreview(double aspectRatio, bool zoom)
+{
+    setGeometry( previewRect );
+    vidArea->setGeometry( calcVideoRectFromAspect(aspectRatio, previewRect, zoom) );
+
+    if (!keyboardGrabbed)
+      keyboardGrabbed = ( myX11GrabKeyboard(getMyWindowId()) == 0 );
+}
+
+void VideoContainer::setPreviewRect(QRect rec)
+{
+   if (rec.height() < 1)
+     rec = QRect(0, 0, 200, 150);
+   
+   previewRect = rec;
+}
+
+void VideoContainer::setScreenRect(QRect rec)
+{
+    if (rec.height() < 1)
+      rec = QRect(0, 0, 400, 300);
+      
+    screenRect = rec;
+}
+
+bool VideoContainer::previouslyZoomed()
+{
+   return lastZoom;
+}
+
+QRect VideoContainer::calcVideoRectFromAspect(double vaspect, QRect targetRect, bool zoom)
+{
+   int vheight = 0;
+   int vwidth = 0;
+   int vdelta = 0;
+   
+   double saspect;
+   QRect vRect;
+
+   if (vaspect == 0)
+     vRect = targetRect;
+   else
+   {
+      saspect = (double)targetRect.width()/targetRect.height();
+      
+   	if ( (vaspect > saspect) != zoom )
+   	{
+			vRect.setWidth( targetRect.width() );
+			vRect.setLeft(0);
+			vheight = int( (double)targetRect.width() / vaspect );
+			vdelta  = int( (targetRect.height() - vheight)/2 );
+			vRect.setTop(vdelta);
+			vRect.setHeight(vheight);
+   	}
+   		else
+   	{
+			vRect.setHeight( targetRect.height() );
+			vRect.setTop(0);
+			vwidth = int( (double)targetRect.height() * vaspect );
+			vdelta = int( (targetRect.width() - vwidth)/2 );
+			vRect.setLeft(vdelta);
+			vRect.setWidth(vwidth);
+   	}
+   }
+
+   return vRect;
 }
 
 // ----------------------------------------------------------------------
@@ -74,6 +191,7 @@ StreamStatus::StreamStatus(QObject *parent)
     proc = NULL;
 
     m_videoContainer = 0;
+    m_fullscreen = Preview;
 
     streamUrl   = "";
     streamName  = "";
@@ -126,8 +244,6 @@ void StreamStatus::timerEvent()
 void StreamStatus::setVideoContainer(VideoContainer *vid)
 {
     m_videoContainer = vid;
-    if ( videoSet() )
-      m_videoContainer->setGeometry( m_videorect );
 }
 
 QString StreamStatus::getVideoWindowId()
@@ -211,11 +327,19 @@ QString StreamStatus::getStreamInfo(StreamInfo info)
     QRegExp reg;
     QString text;
 
+    text = "";
+
     switch (info)
     {
       case StreamVideoFormat:
         text = playerEncap->getStreamProperty("StreamVideoFormat", false);
         break;
+      case StreamVideoWidth:
+        text = playerEncap->getStreamProperty("StreamVideoWidth", false);
+        break;
+      case StreamVideoHeight:
+        text = playerEncap->getStreamProperty("StreamVideoHeight", false);
+        break;
       case StreamLength:
         text = playerEncap->getStreamProperty("StreamLength", false);
         break;
@@ -266,6 +390,25 @@ QString StreamStatus::getStreamInfo(StreamInfo info)
     return text;
 }
 
+double StreamStatus::getVideoAspect()
+{
+   int vheight = 0;
+   int vwidth = 0;
+      
+   QString str;
+
+   str = getStreamInfo(StreamVideoWidth);
+   if (str != "") vwidth = str.toInt();
+   
+   str = getStreamInfo(StreamVideoHeight);
+   if (str != "") vheight = str.toInt();
+   
+   if (vheight == 0)
+     return (double)3/2;
+   else
+     return (double)vwidth/vheight;
+}
+
 void StreamStatus::setStatus(Status stat)
 {
     setIdleOnZero = 0;
@@ -316,14 +459,22 @@ void StreamStatus::setStatus(Status stat)
       }
       case playing:
       {
+        if (oldstatus != paused)
+        {
+				if (video)
+				{
+					videoAspect = getVideoAspect();
+				   m_videoContainer->goPreview(videoAspect, false);
+				}		
+				lastPlayedUrl   = streamUrl;
+				lastPlayedName  = streamName;
+				lastPlayedDescr = streamDescr;
+				lastPlayedHandler = streamHandler;
+				lastPlayedConsole += "\n***" + QString(TARGET": stream playing***\n\n");
+				if (video && videoSet() )
+					myX11MapWindow(m_videoContainer->getMyWindowId());
+        }
         statusString = "playing";
-        lastPlayedUrl   = streamUrl;
-        lastPlayedName  = streamName;
-        lastPlayedDescr = streamDescr;
-        lastPlayedHandler = streamHandler;
-        lastPlayedConsole += "\n***" + QString(TARGET": stream playing***\n\n");
-        if (video && videoSet() )
-          myX11MapWindow(m_videoContainer->getMyWindowId());
         if ( status != oldstatus) emit statusChange();
         break;
       }
@@ -345,7 +496,7 @@ void StreamStatus::setStatus(Status stat)
           m_videoContainer->hide();
         video = false;
         statusString = "stopped";
-        setIdleOnZero = 5;
+        setIdleOnZero = 1;
         lastPlayedConsole += "\n***" + QString(TARGET": stream stopped***\n\n");
         if ( status != oldstatus) emit statusChange();
         break;
@@ -383,14 +534,18 @@ void StreamStatus::setStatus(Status stat)
 
 void StreamStatus::setVideoRect(QRect& rect)
 {
-    m_videorect = rect;
     if ( videoSet() )
-      m_videoContainer->setGeometry( m_videorect );
+    {
+    	m_videoContainer->setPreviewRect(rect);
+    }
 }
 
 void StreamStatus::setVideoMaxRect(QRect& rect)
 {
-   m_videomaxrect = rect;
+	if ( videoSet() )
+	{
+		m_videoContainer->setScreenRect(rect);
+	}
 }
 
 QString& StreamStatus::getStatusString()
@@ -509,7 +664,7 @@ void StreamStatus::reset()
     cacheind = 0;
 
     video = false;
-    if ( m_fullscreen )
+    if ( m_fullscreen != Preview )
       issueCommand(DefaultScreen);
 }
 
@@ -535,15 +690,36 @@ bool StreamStatus::isPending()
     return pending;
 }
 
-
-void StreamStatus::toggleFullScreen()
+// useLastZoom = false: toggles preview -> fullscreen -> fullscreen zoom
+// useLastZoom = true: switch to fullscreen, zoom if previous video was zoomed
+void StreamStatus::toggleFullScreen(bool useLastZoom)
 {
-      if (!m_fullscreen)
-        issueCommand(FullScreen);
-      else
-        issueCommand(DefaultScreen);
+      switch (m_fullscreen)
+      {
+         case Preview:
+           if ( useLastZoom && videoSet() )
+           {
+               m_videoContainer->goFullscreen(videoAspect, m_videoContainer->previouslyZoomed());
+               if ( m_videoContainer->previouslyZoomed() )
+                 m_fullscreen = Zoom;
+               else
+                 m_fullscreen = Full;
+           }
+             else issueCommand(FullScreen);
+         break;
+         case Full:
+           issueCommand(FullScreen);
+         break;
+         case Zoom:
+           issueCommand(DefaultScreen);
+         break;
+      }
 }
 
+bool StreamStatus::isFullScreen()
+{
+   return (m_fullscreen != Preview) && video;
+}
 
 void StreamStatus::issueCommand(StreamCommand command)
 {
@@ -576,17 +752,25 @@ void StreamStatus::issueCommand(StreamCommand command)
     case FullScreen:
       if ( videoSet() )
       {
-        m_videoContainer->setGeometry( m_videomaxrect );
-        playerCommand( playerEncap->getPlayerCmd("fullscreen") );
-        m_fullscreen = true;
+        //playerCommand( playerEncap->getPlayerCmd("fullscreen") );
+        if (m_fullscreen == Full)
+        {
+          m_videoContainer->goFullscreen(videoAspect, true);
+          m_fullscreen = Zoom;
+        }
+        else
+        {
+          m_videoContainer->goFullscreen(videoAspect, false);
+          m_fullscreen = Full;
+        }
       }
       break;
     case DefaultScreen:
       if ( videoSet() )
       {
-        m_videoContainer->setGeometry( m_videorect );
+        m_videoContainer->goPreview(videoAspect, false);
       }
-      m_fullscreen = false;
+      m_fullscreen = Preview;
       break;
   }
 }
@@ -619,8 +803,8 @@ void StreamStatus::startStream()
     {
       proc->addArgument( playerEncap->getPlayerSys("window") );
       proc->addArgument( getVideoWindowId() );
-      proc->addArgument( playerEncap->getPlayerSys("scale") );
-      proc->addArgument( QString::number(m_videorect.width()) );
+      //proc->addArgument( playerEncap->getPlayerSys("scale") );
+      //proc->addArgument( QString::number(m_videorect.width()) );
     }
 
     PlayerEncap::StringMap::Iterator i;
