@@ -34,6 +34,8 @@
 #include "mythstream.h"
 #include "../libs/streambrowser.h"
 
+#include "../libs/speechproc.h"
+
 using namespace std;
 
 // todo: add streamCustomInfo's to audio_panel/video_panel (Artist, Genre, ...)
@@ -206,7 +208,7 @@ void PlayerState::streamStatusChanged(QString status)
 void PlayerState::browserActivityDetected()
 {
    displayStatus = browse;
-   displayResetDelay = 3;
+   displayResetDelay = 4;
 }
 
 /*
@@ -227,7 +229,7 @@ void PlayerState::videoDetected(bool video)
 }
 
 bool PlayerState::pollMe()
-{
+{   
    if (displayResetDelay != -1 && --displayResetDelay == 0)
    {
      if (streamStatus == "playing" || streamStatus == "buffering" )
@@ -257,9 +259,14 @@ MythStream::MythStream(MythMainWindow *parent,
     storageBin = new ReposStorage();
     streamStorage = new StreamStorage("streams", "mythstream" );
     
+    infoTimer = new QTimer( this );
+    connect( infoTimer, SIGNAL(timeout()), this, SLOT(slotGuardInfoField()) );
+    infoTimerRerun = false;
+    
     m_ItemDisplaySize = 7;
     m_FolderDisplaySize = 4;
-
+    commandPending = 0;
+    
     // Load theme and ui file - hacked to introduct UIIconTextType, should be defined in uitypes...
     m_Theme = new XMLParse();
     m_Theme->SetWMult(wmult);
@@ -345,11 +352,11 @@ MythStream::MythStream(MythMainWindow *parent,
 
     connect (
               streamBrowser,
-              SIGNAL(eventUserMessage()),
+              SIGNAL(eventUserMessage(QString&, QString&)),
               this,
-              SLOT(slotUserMessage())
+              SLOT(slotUserMessage(QString&, QString&))
             );
-
+    
     connect (
               streamBrowser,
               SIGNAL( eventValuesUpdated(int) ),
@@ -357,6 +364,16 @@ MythStream::MythStream(MythMainWindow *parent,
               SLOT( slotValuesUpdated(int) )
             );
 
+    connect (
+               streamBrowser,
+               SIGNAL( eventEditString(QString&, QString, bool&) ),
+               this,
+               SLOT  ( slotEditString(QString&, QString, bool&) )
+             );
+                 
+    speechProc = new SpeechProc(this);
+    speechProc->startService();
+    
     itemTreeLoaded = StreamBrowser::streams;
     
     layerSet = m_Theme->GetSet("dyn_panel");
@@ -404,11 +421,165 @@ MythStream::MythStream(MythMainWindow *parent,
 MythStream::~MythStream()
 {
   delete ( streamBrowser );
+
   if (streamStorage)
     delete ( streamStorage );
   if (storageBin)
     delete ( storageBin );
+  if (infoTimer)
+    delete ( infoTimer );
+    
   delete fft;
+  
+  if (speechProc)
+    delete(speechProc);
+}
+
+QString MythStream::popParameterBox(QString myString, QString label, bool &cancelled)
+{
+        QString parameter;
+        int res;
+        
+        popBox = new MythPopupBox(gContext->GetMainWindow(), "");
+        popBox->addLabel(label);
+
+        editBox = new MythLineEdit(popBox, "");
+        editBox->setText(myString);
+        popBox->addWidget(editBox);
+
+        MythPushButton *ok = new MythPushButton(popBox);
+        ok->setText("OK");
+        popBox->addWidget(ok);
+
+        connect(ok, SIGNAL(clicked()), this, SLOT(slotPopResponse()) );
+
+        ok->setFocus();
+        res = popBox->ExecPopup();
+
+        cancelled = (res == -1);
+
+        parameter = editBox->text();
+
+        delete popBox;        
+        popBox = NULL;
+
+        return parameter;
+}
+
+void MythStream::slotPopStreamEditBox(QString &errorMessage)
+{
+    QString fName, iName, uName, dName, hName;
+    QStringList meta;
+
+    errorMessage = "";
+    buttonPushed = NULL;
+    
+    if ( ! streamBrowser->getCurrentStreamObjectDetails(fName, iName, uName, dName, hName, meta) )
+    {
+      errorMessage = "Cannot find stream item";
+      return;
+    }
+        
+		QString parameter;
+		int res;
+		
+		popBox = new MythPopupBox(gContext->GetMainWindow(), "");
+		popBox->addLabel("                  Edit Stream Item                 ");
+
+		MythLineEdit *folderBox = new MythLineEdit(popBox, "");
+		MythLineEdit *nameBox = new MythLineEdit(popBox, "");
+		MythLineEdit *urlBox = new MythLineEdit(popBox, "");
+		MythLineEdit *descrBox = new MythLineEdit(popBox, "");
+		MythLineEdit *handlerBox = new MythLineEdit(popBox, "");
+		
+		folderBox->setText(fName);
+		nameBox->setText(iName);
+		urlBox->setText(uName);
+		descrBox->setText(dName);
+		handlerBox->setText(hName);
+		
+		popBox->addWidget(folderBox);
+		popBox->addWidget(nameBox);
+		popBox->addWidget(urlBox);
+		popBox->addWidget(descrBox);
+		popBox->addWidget(handlerBox);
+
+		MythPushButton *update = new MythPushButton(popBox);
+		update->setText("Update");
+		popBox->addWidget(update);
+		MythPushButton *cancel = new MythPushButton(popBox);
+		cancel->setText("Cancel");
+		popBox->addWidget(cancel);
+		MythPushButton *add = new MythPushButton(popBox);
+		add->setText("Add new");
+		popBox->addWidget(add);
+		MythPushButton *del = new MythPushButton(popBox);
+		del->setText("Delete");
+		popBox->addWidget(del);
+
+		connect(update, SIGNAL(clicked()), this, SLOT(slotPopResponse()) );
+		connect(add, SIGNAL(clicked()), this, SLOT(slotPopResponse()) );
+		connect(del, SIGNAL(clicked()), this, SLOT(slotPopResponse()) );
+		connect(cancel, SIGNAL(clicked()), this, SLOT(slotPopResponse()) );
+
+		cancel->setFocus();
+		res = popBox->ExecPopup();
+
+		if (res != -1)
+		{
+			ValueList oldValues(5);
+			oldValues[0] = fName;
+			oldValues[1] = iName;
+			oldValues[2] = uName;
+			oldValues[3] = dName;
+			oldValues[4] = hName;
+
+			ValueList values(5);
+			values[0] = folderBox->text();
+			values[1] = nameBox->text();
+			values[2] = urlBox->text();
+			values[3] = descrBox->text();
+			values[4] = handlerBox->text();
+
+         if (buttonPushed == (QObject*)update)
+			  streamStorage->updateRecord(ident_maingui, oldValues, values, errorMessage);
+			else if (buttonPushed == (QObject*)add)
+			  streamStorage->insertRecord(ident_maingui, values, errorMessage);
+			else if (buttonPushed == (QObject*)del)
+			  streamStorage->removeRecord(ident_maingui, values, errorMessage);
+		}
+
+		delete popBox;		
+		popBox = NULL;
+
+		return;
+}
+
+void MythStream::slotPopResponse(void)
+{
+   buttonPushed = (QObject*)sender();
+   if (popBox) popBox->done(0);
+}
+
+void MythStream::toggleSpeech()
+{
+    if ( !speechProc->isEnabled() )
+    {
+      speechProc->enable( true );
+      speechProc->say("Welcome to MythStream");
+      speechProc->say("\""+ getCurrentItemName("item") +"\"");
+    }
+      else
+    {
+      speechProc->say("I will be silent. Goodbye");
+      speechProc->enable( false );
+    }
+}
+
+void MythStream::stopSpeech()
+{
+    if ( speechProc->isEnabled() )
+      speechProc->stopSay();
 }
 
 void MythStream::parseContainer(QDomElement& e, QString& name, QRect& area)
@@ -539,7 +710,6 @@ void MythStream::getStreamStatusValues()
 {
 
     QString value, area, temp, containerName;
-    UITextType *ttype;
     QString m_audioCh, m_audioSps, m_audioBps;
 
   // update status_panel fields
@@ -551,7 +721,10 @@ void MythStream::getStreamStatusValues()
     {
       temp = "";
       loadField("status_panel", "message", temp);
+      loadField("status_panel", "custominfo", temp);
 
+      speechProc->say(value);
+      
       playerState.streamStatusChanged(value);
 
       if ( value == "buffering" )
@@ -566,7 +739,7 @@ void MythStream::getStreamStatusValues()
       {
         fft->resetDisplay(); // labels are reset from StreamBrowser
         loadField("status_panel", "custominfo", emptyStr);	
-;
+
 		loadField("audio_panel", "title", emptyStr);
 		loadField("audio_panel", "description", emptyStr);
 		loadField("audio_panel", "audformat", emptyStr);
@@ -669,13 +842,8 @@ void MythStream::getStreamStatusValues()
 //    if (ttype = findTtype(area)) ttype->SetText(value);
 }
 
-void MythStream::getStreamPlayValues()
+void MythStream::checkShowPlayer()
 {
-    QString value, area, title;
-    UITextType *ttype;
-    bool converted;
-    int i;
-
     // reset audio or video display after browse activity
     if ( playerState.pollMe() )
     {
@@ -683,7 +851,17 @@ void MythStream::getStreamPlayValues()
         streamBrowser->showVideo();
       update(m_BotRect);
     }
+}
 
+void MythStream::getStreamPlayValues()
+{
+    QString value, area, title;
+    UITextType *ttype;
+    bool converted;
+    uint i;
+
+	 checkShowPlayer();
+	 
     value = streamBrowser->getStreamParameter(StreamBrowser::streamTime);
     loadField("status_panel", "time", value);
 
@@ -702,7 +880,7 @@ void MythStream::getStreamPlayValues()
     //cout << " volume: " << value << " = int: " << i << endl;
     if (!converted) i = 0;
 
-    if ( i != displayedVolume )
+    if ( (int)i != displayedVolume )
     {
       displayedVolume = i;
       loadBar("dyn_panel", "volumebar", i);
@@ -713,7 +891,7 @@ void MythStream::getStreamPlayValues()
       {
 		if (title && title != "")
 			value = title + ": " + value;
-		if (ttype = findTtype(area))
+		if ( (ttype = findTtype(area)) )
 		  ttype->SetText(value);
 	  }
     
@@ -767,10 +945,9 @@ void MythStream::updateBackground()
 // VideoContainer events
 bool MythStream::eventFilter( QObject *obj, QEvent *ev)
 {
-  // events for VideoContainer, already handled by mplayer
+  // events for VideoContainer
   if (obj && ev->type() == QEvent::KeyPress )
   {
-    // pass key if not handled by mplayer (and how would I know it isn't?)
     QKeyEvent *kev = static_cast<QKeyEvent*>(ev);
     if ( kev->key() != Key_P )
       qApp->sendEvent(this, ev);
@@ -794,14 +971,40 @@ bool MythStream::processAction( QString action )
 {
     Command key_command = nada;
     bool isnumber;   
-    
+
     int actNum = action.toInt(&isnumber);
     if (!isnumber) actNum = -1;
-    
-    if (action == "PREVITEM")    
+     
+    // handle pending commands
+    if ( commandPending == 1 && actNum > -1 && actNum < 10 )
+      {
+        reportEvent("");
+         selectStorageByIndex(storageBin, streamStorage, actNum);
+         commandPending = 0;
+        actNum = -1;
+      }
+    // reset command pending (should be handled above)
+    else if ( commandPending == 1 )
+      {
+        commandPending = 0;
+        actNum = -1;
+        reportEvent("");
+      }
+    // set command pending (0 pressed)
+    else if ( actNum == 0 && commandPending == 0 )
+      {
+       commandPending = 1;
+       actNum = -1;
+       setMessage("Press 0..9 to select storage, other key to cancel");
+      }
+    else if (action == "PREVITEM")
       key_command = previtem;
     else if (action == "NEXTITEM")
       key_command = nextitem;
+    else if (action == "9")
+      key_command = jmpnextitem;
+    else if (action == "3")
+      key_command = jmpprevitem;
     else if (action == "PREVFOLDER")
       key_command = prevfolder;
     else if (action == "NEXTFOLDER")
@@ -816,7 +1019,10 @@ bool MythStream::processAction( QString action )
     else if (action == "ESCAPE")
       key_command = escape;
     else if (action == "END")
+    {
       key_command = end;
+      stopSpeech();
+    }
     else if (action == "DUMP")
       key_command = dump;
     else if (action == "FULLSCREEN")
@@ -849,8 +1055,15 @@ bool MythStream::processAction( QString action )
       key_command = forwardstream;
     else if (action == "REWIND")
       key_command = rewindstream;
-    else if ( actNum > -1 && actNum < 10 )
-      selectStorageByIndex(storageBin, streamStorage, actNum);
+    else if (action == "SPEECH")
+      toggleSpeech();
+    else if (action == "EDITITEM")
+    {
+      QString error;
+      slotPopStreamEditBox(error);
+      if (error != "")
+        reportEvent(error);
+    }
 
     return streamBrowser->handlePressedKey(key_command);
 }
@@ -871,18 +1084,26 @@ void MythStream::keyPressEvent(QKeyEvent *e)
         QString browserAction = "";
                 
         if ( action == "LEFT"  ) browserAction = "PREVFOLDER";
-        if ( action == "RIGHT" ) browserAction = "NEXTFOLDER";
-        if ( action == "UP"    ) browserAction = "PREVITEM";
-        if ( action == "DOWN"  ) browserAction = "NEXTITEM";
+        else if ( action == "RIGHT" ) browserAction = "NEXTFOLDER";
+        else if ( action == "UP"    ) browserAction = "PREVITEM";
+        else if ( action == "DOWN"  ) browserAction = "NEXTITEM";
 
         if (browserAction != "")
         {
           playerState.browserActivityDetected();
-          streamBrowser->hideVideo();
           action = browserAction;
-        }
+          
+          if ( !streamBrowser->videoShown() )
+            handled = processAction(action);
+          else
+          {
+            streamBrowser->hideVideo();      // hide video window
+            updateBotView(true); // oops, gui is mess... (clear video window)
+          }
 
-        handled = processAction(action);        
+        }
+          else
+            handled = processAction(action);    
     }
 
     if (!handled) MythDialog::keyPressEvent(e);
@@ -910,17 +1131,22 @@ void MythStream::slotHarvesterBusy(bool busy, QString message)
     {
       value = "harvesting";
       loadBar("status_panel", "harvestled", 100);
+      speechProc->say(value);
     }
       else
     {
-      // fore browse mode
+      // for browse mode
       playerState.streamStatusChanged("idle");
       update( m_BotRect);
       slotItemTreeSwitchedTo(int(itemTreeLoaded));
+      speechProc->say("idle");
     }
 
     loadField("status_panel", "message", message);
     loadField("status_panel", "status", value);
+    
+    speechProc->say(message);
+
     update( m_TopRect);
 }
 
@@ -931,23 +1157,52 @@ void MythStream::slotVideoActive(bool active)
    update( m_BotRect );
 }
 
-void MythStream::slotUserMessage()
+void MythStream::slotGuardInfoField()
 {
-  QString value = streamBrowser->getUserMessage();
-  loadField("status_panel", "message", value);
-  update( m_TopRect );
+   if (infoTimerRerun)
+   {
+      infoTimer->start( 5000, TRUE );
+      infoTimerRerun = false;
+      return;
+   }
+   
+   QString info = "";
+
+   loadField("status_panel", "message", info);
+   loadField("status_panel", "custominfo", info);
+   
+   update( m_TopRect );
 }
 
-void MythStream::reportEvent(QString msg)
+void MythStream::slotUserMessage(QString &msg, QString &info)
 {
   loadField("status_panel", "message", msg);
+  loadField("status_panel", "custominfo", info);
+     
+  if ( !infoTimer->isActive() )
+    infoTimer->start( 5000, TRUE );
+  else
+    infoTimerRerun = true;
+  
   update( m_TopRect );
-/*
-   userMessageBox->setText( msg );
-   userMessageBox->show();
-*/
 }
 
+// temp. message, clears after 4 sec
+void MythStream::reportEvent(QString msg)
+{
+  QString info = "";
+  slotUserMessage(msg, info);
+}
+
+// permanent message
+void MythStream::setMessage(QString msg)
+{
+  QString info = "";
+  infoTimer->stop();
+  loadField("status_panel", "message", msg);
+  loadField("status_panel", "custominfo", info);
+  update( m_TopRect );
+}
 
 void MythStream::getFolderList()
 {
@@ -965,6 +1220,11 @@ void MythStream::getFolderList()
       loadIconSource("browse_panel", "folder_right", emptyPict);
     else
       loadIconSource("browse_panel", "folder_right", rightPict);
+
+// speech test
+    if ( ( cursor < (int)list.count() ) && cursor >= 0 )
+      speechProc->say("\"folder: "+ list[cursor] +"\"");
+
 }
 
 
@@ -992,8 +1252,34 @@ void MythStream::getItemList()
     }
     
     loadListFields("browse_panel", "item", list, cursor);
+    
+// speech test
+    if ( ( cursor < (int)list.count() ) && cursor >= 0 )
+      speechProc->say("\""+ list[cursor].remove(0,1) +"\"");
+    
 }
 
+QString MythStream::getCurrentItemName(QString type)
+{
+    QString fName, iName, uName, dName, hName;
+    QStringList meta;
+    
+    if ( ! streamBrowser->getCurrentStreamObjectDetails(fName, iName, uName, dName, hName, meta) )
+    {
+      iName = "";
+      fName = "";
+    }
+
+    if (type == "item")
+      return iName;
+    else
+      return fName;
+}
+
+void MythStream::slotEditString(QString &line, QString label, bool &cancelled)
+{
+   line = popParameterBox(line, label, cancelled);
+}
 
 void MythStream::slotValuesUpdated(int updatedType)
 {
@@ -1006,9 +1292,12 @@ void MythStream::slotValuesUpdated(int updatedType)
      break;
      case StreamBrowser::streamplay:
        getStreamPlayValues();
-       update(m_BotRect);  // stream play status
+       update(m_MidRect);  
        update(m_TopRect);  // stream play status
      break;
+     case StreamBrowser::fsstreamplay:
+       checkShowPlayer();
+     break;
      case StreamBrowser::folderlist:
        getFolderList();
        getItemList();
@@ -1052,7 +1341,7 @@ UITextType * MythStream::findTtype(const QString field)
 {
   // find field in status_panel or audio_panel containers
   QString container = "status_panel";
-  UITextType *ttype;
+  UITextType *ttype = NULL;
   LayerSet* layerSet;
 
   layerSet = m_Theme->GetSet(container);
@@ -1254,6 +1543,7 @@ void MythStream::loadBar(const QString& container, const QString& field, int val
 
 void MythStream::updateTopView()
 {
+    
    // unconditional update
     QPixmap pix(m_TopRect.size());
     pix.fill(this, m_TopRect.topLeft());
@@ -1278,6 +1568,7 @@ void MythStream::updateTopView()
 
     bitBlt(this, m_TopRect.left(), m_TopRect.top(),
            &pix, 0, 0, -1, -1, Qt::CopyROP);
+    
 }
 
 
@@ -1301,14 +1592,16 @@ void MythStream::updateMidView()
     }
 
     p.end();
-
+    
     bitBlt(this, rect.left(), rect.top(),
            &pix, 0, 0, -1, -1, Qt::CopyROP);
+    
 }
 
 
 void MythStream::updateBotView(bool clearOnly)
 {
+    
     LayerSet* container = 0;
     
     switch ( playerState.getDisplayStatus() )
@@ -1333,7 +1626,7 @@ void MythStream::updateBotView(bool clearOnly)
         break;
     }
 
-   if (clearOnly) // todo
+   if (clearOnly) // clear video panel when switching to (smaller) browse window
      container = m_Theme->GetSet("video_panel");
 
     QRect rect = container->GetAreaRect();
@@ -1360,9 +1653,10 @@ void MythStream::updateBotView(bool clearOnly)
     }
 
     p.end();
-
+        
     bitBlt(this, rect.left(), rect.top(),
            &pix, 0, 0, -1, -1, Qt::CopyROP);
+    
 }
 
 void MythStream::slotWebStorageMaybeReady()
@@ -1370,7 +1664,8 @@ void MythStream::slotWebStorageMaybeReady()
    QString error;
    
    if ( !streamStorage->loadList(100, error) )
-        cerr << error << endl;
+   if (error != "")
+     reportEvent(error);
 
 }
 void MythStream::selectStorageByIndex(ReposStorage *storageBin, StreamStorage *streamStorage, int index)
@@ -1382,10 +1677,12 @@ void MythStream::selectStorageByIndex(ReposStorage *storageBin, StreamStorage *s
     bool found = false;
     
     storageBin->resetRecordList();
+
+    reportEvent("");
     
-    while ( i != index && i < 9 && (found = storageBin->getNextRecord(values)) )
+    while ( i != index && i < 8 && (found = storageBin->getNextRecord(values)) )
     {
-      cout << values[3] << endl;
+      //cout << values[3] << endl;
       i++; 
     }
 
@@ -1396,9 +1693,15 @@ void MythStream::selectStorageByIndex(ReposStorage *storageBin, StreamStorage *s
           if ( !streamStorage->loadList(100, error) )
             cerr << error << endl;
       break;
-    
+          
+      case 8:
+          streamStorage->selectWebStorage(
+             0, "online demo", "http://home.kabelfoon.nl/~moongies/harvester/streams"VERSION".res", "", "");
+          QTimer::singleShot( 600, this, SLOT(slotWebStorageMaybeReady()) );
+      break;
+      
       case 9:
-          streamStorage->selectFileStorage(0, " from v"VERSION" tarball", PREFIX"/share/"SUBPATH"/streams.res");
+          streamStorage->selectFileStorage(0, "v"VERSION" tarball demo", PREFIX"/share/"SUBPATH"/streams.res");
           if ( !streamStorage->loadList(100, error) )
             cerr << error << endl;
       break;
@@ -1408,6 +1711,8 @@ void MythStream::selectStorageByIndex(ReposStorage *storageBin, StreamStorage *s
           {
             reportEvent("");
             storageBin->openStorage(streamStorage, ident_storagegroup, values, error);
+            if (error != "")
+              reportEvent(error);
             QTimer::singleShot( 600, this, SLOT(slotWebStorageMaybeReady()) );
           }
             else reportEvent("Stream storage index " + QString::number(index) + " not configured");
